@@ -1,7 +1,9 @@
 use anyhow::{anyhow, bail, Result};
 use quote::ToTokens;
 use std::collections::HashMap;
+use syn::ReturnType;
 
+use crate::helpers::get_docs;
 use crate::{
     function::{self, ParserType},
     impl_::{parse_attribute, ParsedAttribute, PropAttrTy, RenameRule, Visibility},
@@ -31,6 +33,7 @@ pub struct Method {
     pub ident: String,
     /// Rust internal function ident
     pub orig_ident: String,
+    pub docs: Vec<String>,
     pub args: Vec<Arg>,
     pub optional: Option<String>,
     pub output: Option<(String, bool)>,
@@ -68,40 +71,48 @@ impl ParsedMethod {
     }
 }
 
-pub fn parser(mut input: ImplItemMethod, rename_rule: RenameRule) -> Result<ParsedMethod> {
+pub fn parser(
+    struct_ty: &Type,
+    mut input: ImplItemMethod,
+    rename_rule: RenameRule,
+) -> Result<ParsedMethod> {
     let mut defaults = HashMap::new();
     let mut optional = None;
     let mut visibility = Visibility::Public;
     let mut as_prop = None;
     let mut identifier = None;
     let mut is_constructor = false;
+    let docs = get_docs(&input.attrs);
 
     for attr in input.attrs.iter() {
-        match parse_attribute(attr)? {
-            ParsedAttribute::Default(list) => defaults = list,
-            ParsedAttribute::Optional(name) => optional = Some(name),
-            ParsedAttribute::Visibility(vis) => visibility = vis,
-            ParsedAttribute::Rename(ident) => identifier = Some(ident),
-            ParsedAttribute::Property { prop_name, ty } => {
-                if as_prop.is_some() {
-                    bail!(
-                        "Only one `#[getter]` and/or `#[setter]` attribute may be used per method."
-                    );
+        if let Some(attr) = parse_attribute(attr)? {
+            match attr {
+                ParsedAttribute::Default(list) => defaults = list,
+                ParsedAttribute::Optional(name) => optional = Some(name),
+                ParsedAttribute::Visibility(vis) => visibility = vis,
+                ParsedAttribute::Rename(ident) => identifier = Some(ident),
+                ParsedAttribute::Property { prop_name, ty } => {
+                    if as_prop.is_some() {
+                        bail!(
+                            "Only one `#[getter]` and/or `#[setter]` attribute may be used per method."
+                        );
+                    }
+
+                    let prop_name = prop_name.unwrap_or_else(|| {
+                        input
+                            .sig
+                            .ident
+                            .to_token_stream()
+                            .to_string()
+                            .trim_start_matches("get_")
+                            .trim_start_matches("set_")
+                            .to_string()
+                    });
+                    as_prop = Some((prop_name, ty))
                 }
-                let prop_name = prop_name.unwrap_or_else(|| {
-                    input
-                        .sig
-                        .ident
-                        .to_token_stream()
-                        .to_string()
-                        .trim_start_matches("get_")
-                        .trim_start_matches("set_")
-                        .to_string()
-                });
-                as_prop = Some((prop_name, ty))
+                ParsedAttribute::Constructor => is_constructor = true,
+                _ => bail!("Invalid attribute for method."),
             }
-            ParsedAttribute::Constructor => is_constructor = true,
-            _ => bail!("Invalid attribute for method."),
         }
     }
 
@@ -123,7 +134,7 @@ pub fn parser(mut input: ImplItemMethod, rename_rule: RenameRule) -> Result<Pars
         quote! { return; }
     };
     let internal_ident = Ident::new(&format!("_internal_php_{}", ident), Span::call_site());
-    let args = build_args(&mut input.sig.inputs, &defaults)?;
+    let args = build_args(struct_ty, &mut input.sig.inputs, &defaults)?;
     let optional = function::find_optional_parameter(
         args.iter().filter_map(|arg| match arg {
             Arg::Typed(arg) => Some(arg),
@@ -193,9 +204,10 @@ pub fn parser(mut input: ImplItemMethod, rename_rule: RenameRule) -> Result<Pars
         name,
         ident: internal_ident.to_string(),
         orig_ident: ident.to_string(),
+        docs,
         args,
         optional,
-        output: crate::function::get_return_type(&input.sig.output)?,
+        output: get_return_type(struct_ty, &input.sig.output)?,
         _static: matches!(method_type, MethodType::Static),
         visibility,
     };
@@ -203,7 +215,84 @@ pub fn parser(mut input: ImplItemMethod, rename_rule: RenameRule) -> Result<Pars
     Ok(ParsedMethod::new(func, method, as_prop, is_constructor))
 }
 
+pub fn get_return_type(self_ty: &Type, output_type: &ReturnType) -> Result<Option<(String, bool)>> {
+    Ok(match output_type {
+        ReturnType::Default => None,
+        ReturnType::Type(_, ty) => {
+            let mut ty = ty.clone();
+            replace_self(self_ty, &mut ty);
+            crate::function::Arg::from_type("".to_string(), &ty, None, true)
+                .map(|arg| (arg.ty, arg.nullable))
+        }
+    })
+}
+
+/// Takes a type `ty` and replaces all instances of `Self` with the type
+/// `self_ty`.
+fn replace_self(self_ty: &Type, ty: &mut Type) {
+    match ty {
+        Type::Array(syn::TypeArray { elem, .. }) => replace_self(self_ty, elem),
+        Type::BareFn(syn::TypeBareFn { inputs, output, .. }) => {
+            for input in inputs {
+                replace_self(self_ty, &mut input.ty);
+            }
+            if let ReturnType::Type(_, ty) = output {
+                replace_self(self_ty, ty);
+            }
+        }
+        Type::Group(syn::TypeGroup { elem, .. }) => replace_self(self_ty, elem),
+        Type::Paren(syn::TypeParen { elem, .. }) => replace_self(self_ty, elem),
+        Type::Path(syn::TypePath { qself, path }) => {
+            if let Some(syn::QSelf { ty, .. }) = qself {
+                replace_self(self_ty, ty);
+            }
+            for seg in &mut path.segments {
+                if seg.ident == "Self" {
+                    seg.ident =
+                        Ident::new(&self_ty.to_token_stream().to_string(), Span::call_site());
+                }
+                match &mut seg.arguments {
+                    syn::PathArguments::AngleBracketed(syn::AngleBracketedGenericArguments {
+                        args,
+                        ..
+                    }) => {
+                        for arg in args {
+                            if let syn::GenericArgument::Type(ty) = arg {
+                                replace_self(self_ty, ty);
+                            }
+                        }
+                    }
+                    syn::PathArguments::Parenthesized(syn::ParenthesizedGenericArguments {
+                        inputs,
+                        output,
+                        ..
+                    }) => {
+                        for input in inputs {
+                            replace_self(self_ty, input);
+                        }
+
+                        if let ReturnType::Type(_, ty) = output {
+                            replace_self(self_ty, ty);
+                        }
+                    }
+                    _ => {}
+                }
+            }
+        }
+        Type::Ptr(syn::TypePtr { elem, .. }) => replace_self(self_ty, elem),
+        Type::Reference(syn::TypeReference { elem, .. }) => replace_self(self_ty, elem),
+        Type::Slice(syn::TypeSlice { elem, .. }) => replace_self(self_ty, elem),
+        Type::Tuple(syn::TypeTuple { elems, .. }) => {
+            for elem in elems {
+                replace_self(self_ty, elem);
+            }
+        }
+        _ => {}
+    }
+}
+
 fn build_args(
+    struct_ty: &Type,
     inputs: &mut Punctuated<FnArg, Token![,]>,
     defaults: &HashMap<String, Lit>,
 ) -> Result<Vec<Arg>> {
@@ -222,9 +311,11 @@ fn build_args(
                 let mut this = false;
                 let attrs = std::mem::take(&mut ty.attrs);
                 for attr in attrs.into_iter() {
-                    match parse_attribute(&attr)? {
-                        ParsedAttribute::This => this = true,
-                        _ => bail!("Invalid attribute for argument."),
+                    if let Some(attr) = parse_attribute(&attr)? {
+                        match attr {
+                            ParsedAttribute::This => this = true,
+                            _ => bail!("Invalid attribute for argument."),
+                        }
                     }
                 }
 
@@ -236,8 +327,11 @@ fn build_args(
                         _ => bail!("Invalid parameter type."),
                     };
                     let default = defaults.get(&name);
+                    let mut ty = ty.ty.clone();
+                    replace_self(struct_ty, &mut ty);
+
                     Ok(Arg::Typed(
-                        crate::function::Arg::from_type(name.clone(), &ty.ty, default, false)
+                        crate::function::Arg::from_type(name.clone(), &ty, default, false)
                             .ok_or_else(|| anyhow!("Invalid parameter type for `{}`.", name))?,
                     ))
                 }
